/*
 * @(#)AliceTest.java           May 2001
 *
 * This is the main servlet for the AIM system.
 *
 *
 * The home page for any AIM installation contains a form like this
 *
 *  <form method="post" action="servlets/Alice">...</form>
 *
 * The web pages created dynamically by the system also contain similar
 * forms.  Whenever such a form is submitted, the 'doPost' method of this
 * servlet is invoked to process the form.  The data entered in the form 
 * by the user are passed to the servlet in the 'request' argument of the
 * 'doPost' method.  For example, if the form contains an element like
 *
 *  Your Student ID: <input type="text" name="StudentID">
 *
 * which the student uses to enter an ID of '197453' then in the body of
 * the 'doPost' method, the expression
 *
 *  request.getParameterValues("StudentID")
 *
 * would evaluate to the string "197453".  Note that the AIM-generated
 * pages typically contain 'hidden' elements like
 * 
 *  <input type="hidden" name="Subject" value="Calculus">
 *
 * These elements are not visible in the user's browser, but they still
 * get passed to the servlet, so that
 * 
 *  request.getParameterValues("Subject")
 *
 * would evaluate to the string "Calculus".  Essentially, all that
 * this servlet does is to pass all these parameters to Maple.  In
 * more detail, if the form sets parameters a,b,c to values "Alpha",
 * "Beta" and "Gamma" then the command sent to Maple will be:
 *
 *  `aim/Main`("a" = "Alpha", "b" = "Beta", "c" = "Gamma");
 *
 * Note that both the parameter and its value are passed as strings.
 * This seems the safest way to ensure that Maple makes no attempt to
 * parse or evaluate them until it is sure that it wants to.
 * (Evaluating arbitrary strings supplied by an unknown user is of
 * course a security nightmare.)
 *
 * In response to whatever command is sent, Maple will print a web
 * page, which the servlet passes back to the user.  Generally speaking,
 * the page generated by Maple will include a form like
 *
 *  <form method="post" action="servlets/Alice">...</form>
 *
 * so when the form is submitted, control will pass back to the
 * servlet again.  Moreover, the form will include elements like
 *
 *  <input type="hidden" name="StudentID" value="854584">
 *
 * (and similarly for subject name, quiz name etc) so that the user
 * does not have to say repeatedly who s/he is or which quiz s/he
 * wants to do.
 *
 ****
 *
 * The main subtlety is in the management of Maple processes.  Every
 * time we start a new process, it has to initialize itself by loading
 * the AIM software, which is slow.  We therefore want to reuse
 * processes as far as possible.  We therefore maintain a pool of
 * Maple processes, and keep track of which of them are in use.  This
 * is handled by an object of class MaplePool;  each individual
 * process is represented by an object of class Maple.
 *
 ****
 *
 * The servlet requires a number of initialization parameters, such 
 * as the path to the Maple executable and so on.  These parameters
 * are read from a file WEB-INF/web.xml under the main AIM directory.
 * This file is generated automatically and should not generally be
 * edited by hand; see the AIM installation instructions for details. 
 *
 ****
 *
 * The servlet records each request, and some other events, in a log
 * file, whose location  is one of the initialization parameters.
 * It also records the most recent request in a separate file in a
 * slightly different format, which is convenient for debugging.
 * There is a separate log file for TeX-to-HTML translation problems,
 * which is currently not used.
 */

//Import needed Java packages

import java.io.*;
import java.util.*;
import java.text.DateFormat;
import javax.servlet.*;
import javax.servlet.http.*;
import com.oreilly.servlet.*;

/******************************************************************
  Create a class which inherits from GenericServlet or HttpServlet.
******************************************************************/
public class Alice extends HttpServlet
{ 
  // This field is a class variable.  If several users submit forms
  // simultaneously, they will each have their own thread of the
  // servlet, but all instances will share the maplePool.
  private static MaplePool maplePool;

  private static AliceLog aliceLog;

  private String AliceMapleDir;
  // The directory containing the Maple source files Aim.mpl etc

  private String TempDir;
  // The directory used to store uploaded files.  These are deleted
  // after the relevant Maple command has been executed, so the 
  // Maple command is expected to move them to a more permanent place
  // if necessary.

  private int MaxPostSize;
  // The maximum allowed size for uploaded files, in bytes

  private String MapleProgram;
  // The command used to start Maple

  private String MapleInitOptions;
  // Initialization options to pass to Maple

  private int MaxMapleProcesses;
  // The maximum allowed number of maple processes

  private String LogFile;
  // The name of a file in which to log all commands sent to Maple.
  // If this is null, no logging is done.

  private String LastRequestFile;
  // The name of a file in which to log the parameter list
  // for the last command sent to Maple.
  // Usually, this will also be null and no logging will be done. 

// KM
  private String KillProgram;
  private String KillProgramArg;
  // These form the command to kill processes

  private int AliceHistorySize;
  // the number of "Recent Requests" to list on the Alice Monitor page

  private int StudentTimeLimit;
  // the time out value for non-administrative commands (milliseconds)

  private int AdminTimeLimit;
  // the time out value for administrative commands (milliseconds)

  public boolean MapleFinished = false;
  // A flag set by Maple when it finishes correctly
// END KM

  private String TtHProgram;
  // The command to be sent to TtH

  private String TtHLogFile;
  // The name of a file in which to log TtH problems

  private int TtHLogLevel;
  // This controls how much logging information is generated by TtH

  private String BusyMessage;
  // The message to print if the server is too busy.

  private String configError;
  // This is null if there were no configuration problems

  //////////////////////////////////////////////////////////////////////

  // Generally speaking, the init method will only be run once, the
  // first time that servlet is accessed after the web server has been
  // started up.
  public void init(ServletConfig config)
    throws ServletException
  {
    super.init(config);

// KM
    try {
      AliceHistorySize =Integer.parseInt(getInitParameter("AliceHistorySize"));
    } catch(Exception ex) {
      AliceHistorySize = 20;
    }

// dist    aliceLog = new AliceLog(20);
    aliceLog = new AliceLog(AliceHistorySize);
// END KM

    MapleProgram = getInitParameter("MapleProgram");
    MapleInitOptions = getInitParameter("MapleInitOptions");

    AliceMapleDir = getInitParameter("AliceMapleDir");
    TempDir = getInitParameter("TempDir");

    // It doesn't necessarily matter if these ones are null

    try {
      MaxMapleProcesses =
        Integer.parseInt(getInitParameter("MaxMapleProcesses"));
    } catch(Exception ex) {
      MaxMapleProcesses = 8;
    }

    try {
      MaxPostSize =
        Integer.parseInt(getInitParameter("MaxPostSize"));
    } catch(Exception ex) {
      MaxPostSize = 10 * 1024 * 1024;
    }

    LogFile = getInitParameter("LogFile");
    LastRequestFile = getInitParameter("LastRequestFile");

// KM
    KillProgram = getInitParameter("KillProgram");
    KillProgramArg = getInitParameter("KillProgramArg");

    try {
      StudentTimeLimit =Integer.parseInt(getInitParameter("StudentTimeLimit"));
    } catch(Exception ex) {
      StudentTimeLimit = 0;
    }

    try {
      AdminTimeLimit =Integer.parseInt(getInitParameter("AdminTimeLimit"));
    } catch(Exception ex) {
      AdminTimeLimit = 0;
    }
// END KM

    TtHProgram = getInitParameter("TtHProgram");
    TtHLogFile = getInitParameter("TtHLogFile");
    try {
      TtHLogLevel =
        Integer.parseInt(getInitParameter("TtHLogLevel"));
    } catch(Exception ex) {
      TtHLogLevel = 1;
    }

    BusyMessage = getInitParameter("BusyMessage");
    if (BusyMessage == null) {
      BusyMessage = 
        "The Alice server is busy.  Please try again in a few minutes.";
    }

    if (TtHProgram != null) {
      tth.TtH.program = TtHProgram;
    }
    tth.TtH.logFile = TtHLogFile;
    tth.TtH.logLevel = TtHLogLevel;

    checkConfig();

    if (configError == null) {
      writeLog("Alice servlet initialized\n" + ParamReport() + "\n");

      // Start the Maple process pool
      if (maplePool == null) {
        maplePool = new MaplePool();
        maplePool.program = MapleProgram;
        maplePool.initOptions = MapleInitOptions;

// KM
        maplePool.killProgram = KillProgram;
        maplePool.killProgramArg = KillProgramArg;
// END KM

        maplePool.maxProcesses = MaxMapleProcesses;
        maplePool.logFile = LogFile;

        writeLog("Maple process pool started\n");
      }
    } else {
      writeLog("Alice servlet not initialized because of " +
               "configuration problems: \n\n " + 
               configError + "\n\n");
    }
  }
  
  private void checkConfig() {
    Vector missing = new Vector();
    String s ;
    StringWriter err_ = new StringWriter();
    PrintWriter err = new PrintWriter(err_);
    File f;

    if (AliceMapleDir == null) {
      missing.addElement("AliceMapleDir");
    } else {
      AliceMapleDir = AliceMapleDir.trim();
      f = new File(AliceMapleDir);
      s = "";
      if (f.isAbsolute()) {
        if (f.exists()) {
          if (f.isDirectory()) {
            // OK
          } else {
            s = "This is not a directory.";
          }
        } else {
          s = "This does not exist.";
        }
      } else {
        s = "This is not an absolute path.";
      }
      if (s.length() > 0) {
        err.println("<p>");
        err.println("AliceMapleDir was set to '" + AliceMapleDir + "'");
        err.println("<br>");
        err.println(s);
        err.println("</p>");
      } else {
        f = new File(f,"lib");
        f = new File(f,"Aim.m");
        if (f.exists()) {
          // OK
        } else {
          err.println("<p>");
          err.println("AliceMapleDir was set to '" + AliceMapleDir + "'");
          err.println("<br>");
          err.println(f.getPath() +
                      " does not exist.  This means that the Maple " +
                      "source files for AIM have not been compiled " +
                      "(by running Make.mpl) or are not in the " +
                      "right place.");
          err.println("</p>");
        }
      }
    }

    if (TempDir == null) {
      missing.addElement("TempDir");
    } else {
      TempDir = TempDir.trim();
      f = new File(TempDir);
      s = "";
      if (f.isAbsolute()) {
        if (f.exists()) {
          if (f.isDirectory()) {
            if (f.canWrite()) {
              // OK
            } else {
              s = "The servlet cannot write to this directory.";
            }
          } else {
            s = "This is not a directory.";
          }
        } else {
          s = "This does not exist.";
        }
      } else {
        s = "This is not an absolute path.";
      }
      if (s.length() > 0) {
        err.println("<p>");
        err.println("TempDir was set to '" + TempDir + "'");
        err.println("<br>");
        err.println(s);
        err.println("</p>");
      }
    }

// KM
    if (KillProgram == null) {
      missing.addElement("KillProgram");
    } else {
      KillProgram = KillProgram.trim();
      f = new File(KillProgram);
      s = "";
      if (f.isAbsolute()) {
        if (f.exists()) {
          if (f.isFile()) {
            // OK
          } else {
            s = "This exists, but is not a regular file.";
          }
        } else {
          s = "This does not exist.";
        }
      } else {
        s = "This is not an absolute path.";
      }
      if (s.length() > 0) {
        err.println("<p>");
        err.println("KillProgram was set to '" + KillProgram + "'");
        err.println("<br>");
        err.println(s);
        err.println("</p>");
      }
    }
// END KM

    if (TtHProgram == null) {
      missing.addElement("TtHProgram");
    } else {
      TtHProgram = TtHProgram.trim();
      f = new File(TtHProgram);
      s = "";
      if (f.isAbsolute()) {
        if (f.exists()) {
          if (f.isFile()) {
            // OK
          } else {
            s = "This exists, but is not a regular file.";
          }
        } else {
          s = "This does not exist.";
        }
      } else {
        s = "This is not an absolute path.";
      }
      if (s.length() > 0) {
        err.println("<p>");
        err.println("TtHProgram was set to '" + TtHProgram + "'");
        err.println("<br>");
        err.println(s);
        err.println("</p>");
      } else {
        s = tth.TtH.test();
        if (s != null) {
          err.print(s);
        }
      }
    }

    if (MapleProgram == null) {
      missing.addElement("MapleProgram");
    } else {
      MapleProgram = MapleProgram.trim();
      f = new File(MapleProgram);
      s = "";
      if (f.isAbsolute()) {
        if (f.exists()) {
          if (f.isFile()) {
            // OK
          } else {
            s = "This exists, but is not a regular file.";
          }
        } else {
          s = "This does not exist.";
        }
      } else {
        s = "This is not an absolute path.";
      }
      if (s.length() > 0) {
        err.println("<p>");
        err.println("MapleProgram was set to '" + MapleProgram + "'");
        err.println("<br>");
        err.println(s);
        err.println("</p>");
      }
    }

    if (MapleInitOptions != null &&
        MapleInitOptions.trim().equalsIgnoreCase("default")) {
      MapleInitOptions = null;
    }

    if (MaxMapleProcesses < 1) {
      err.println("<p>");
      err.println("MaxMapleProcesses was set to " + MaxMapleProcesses);
      err.println("<br>");
      err.println("Values less than one are not sensible.");
      err.println("</p>");
    }

    checkLogFile("LogFile",        LogFile,        err);
    checkLogFile("LastRequestFile",LastRequestFile,err);
    checkLogFile("TtHLogFile",     TtHLogFile,     err);

    if (!missing.isEmpty()) {
      if (missing.size() == 1) {
        err.println("<p>");
        err.println("The required parameter " + 
                    (String)(missing.firstElement()) +
                    " is missing.");
        err.println("</p>");
      } else {
        err.println("<p>");
        err.println("The following required parameters are missing:");
        err.println("<br>");
        Enumeration e = missing.elements();
        while (e.hasMoreElements()) {
          String m = (String) e.nextElement();
          err.println(m);
        }
        err.println("</p>");    
      }
    }

    configError = err_.toString();
    if (configError.length() == 0) {
      configError = null;
    } else {
      configError =
        "<html>\n" +
        "<head><title>Configuration error report</title></head>\n" +
        "<body bgcolor='white'>\n" +
        "<h1>Alice server configuration error report</h1>             \n" +
        "The Alice server did not start, because of configuration     \n" +
        "problems.                                                    \n" +
        "<hr noshade>\n" +
        configError +
        "<hr noshade>\n<p>\n" +
        "<b>Note:</b><br>\n" +
        "These messages refer to initialization parameters set in     \n" +
        "the file \n<br>\n" +
        "<font color='green'>" + 
        "<tt>&lt;top AIM directory&gt;/WEB-INF/web.xml</tt>" +
        "</font>" + 
        ". \n<br>\n" +
        "If you need to change these parameters, the approved         \n" +
        "procedure is to create or edit " +
        "<font color='green'><tt>ManualConfig.mpl</tt></font>,\n" +
        "then run \n" +
        "<font color='green'><tt>AutoConf.mpl</tt></font>\n" +
        " and \n" +
        "<font color='green'><tt>Servlet.mpl</tt></font>\n" +
        "to regenerate \n" +
        "<font color='green'><tt>web.xml</tt></font>\n" +
        "automatically.\n" +
        "See the AIM installation documentation for more details.     \n" +
        "</p>\n";
    }
  }
  
  private void checkLogFile(String n,String l,PrintWriter p) {
    String s;
    File f,d;

    if (l != null) {
      f = new File(l);
      s = "";
      if (f.isAbsolute()) {
        if (f.exists()) {
          if (f.isFile()) {
            if (f.canWrite()) {
              // OK
            } else {
              s = "The servlet cannot write to this file.";
            }
          } else {
            s = "This is not a regular file.";
          }
        } else {
          // does not exist (yet)
          d = new File(f.getParent());
          if (d.exists()) {
            if (d.canWrite()) {
              // OK
            } else {
              s = "This does not exist, and the servlet cannot create it.";
            }
          } else {
            s = "The parent directory does not exist.";
          }
        }
      } else {
        s = "This is not an absolute path.";
      }
      if (s.length() > 0) {
        p.println(n + " was set to '" + LogFile + "'");
        p.print("<br>\n" + s + "\n<br>\n");
      }
    }    
  }

  public void doGet (HttpServletRequest request,
                     HttpServletResponse response)
    throws ServletException, IOException
  {
    doPost(request,response);
  }

  public void doPost (HttpServletRequest request,
                      HttpServletResponse response)
    throws ServletException, IOException
  {
    // This is for output to be sent as our response.
    StringWriter webOutput_ = new StringWriter();
    PrintWriter webOutput = new PrintWriter(webOutput_);

    // This if for reporting error messages
    StringWriter msg_;
    PrintWriter msg;

    PrintWriter logWriter;

    Maple maple;
    String commandparam,initcommand,commandargs,page;

    // logRequest(request);

    if (configError != null) {
      response.setContentType("text/html");
      response.getWriter().print(configError);
      response.getWriter().close();
      return;
    }

    AliceRequest alicereq =
      new AliceRequest(request,TempDir,MaxPostSize);

    commandparam = alicereq.getParameter("Command");
    if ((commandparam != null) && 
        commandparam.startsWith("Servlet")) {
      // this request is to be handled directly by the
      // servlet, not by Maple.

      doServletCommand(commandparam,request,response);
      return;
    }

    AliceLogEntry logentry = new AliceLogEntry(alicereq);
    aliceLog.addEntry(logentry);

    // Build the command to send to Maple
    commandargs = alicereq.mapleCommandArgs();

    // Get a Maple process to execute the command
    maple = null;

    try { 

      maple = maplePool.getMaple(alicereq.studentID);

    } catch (Exception e) {
      // maple pool can't get a maple process error
      msg_ = new StringWriter();
      msg = new PrintWriter(msg_);

      msg.write("Fatal error getting new Maple process:\n" +
                "Exception message:\n" + e.getMessage() + "\n\n" +
                "Stack trace:\n");

      e.printStackTrace(msg);
      
      msg.write("\n\n");

      throw(new UnavailableException(msg_.toString()));
    }

    // No maple processes are available... the server is busy
    if (maple == null) {
      printBusyPage(response);
      logentry.close("SERVER BUSY");
      return;
    }

    logentry.processID = maple.id;

    initMaple(maple);

// KM
// We now pass the IP address of the requestor to both the log file and
// to Maple.

    String mapleCmd;
    String ipCmdArgs;
    if (commandargs.equals("")) {
       ipCmdArgs = "\"IPAddress\"=" + Maple.quote(request.getRemoteAddr());
    } else {
       ipCmdArgs = commandargs + ",\"IPAddress\"=" + Maple.quote(request.getRemoteAddr());
    }
    mapleCmd = "`aim/Main`(" + ipCmdArgs + "):\n";

    writeLog("Command sent to maple process " + 
             maple.id + " from " + request.getRemoteAddr() +
             ": " + mapleCmd + "\n");

// dist writeLog("Command sent to maple process " + 
// dist   maple.id + ":\n" + "`aim/Main`(" + commandargs + "):\n\n");
// end KM

    if (LastRequestFile != null) {
      try {
        logWriter =
          new PrintWriter(new FileWriter(LastRequestFile));
        logWriter.println("paramlist := [" + ipCmdArgs + "]:");
        logWriter.println("");
        logWriter.close();
      } catch( Exception e ) {
        throw new ServletException("Could not write last request file\n",e);
      }
    }

    // execute command, sending output to the web page.
// KM
// dist   try {
// dist   maple.exec("`aim/Main`(" + commandargs + "):\n", webOutput);
      // Decide what the timeout value should be for this command
      // Student commands get less time than administrator commands
      int timeout = timeLimit(commandparam);
      String logentryMsg = null;

      // Declare a separate thread to run the Maple command
      MapleCommandThread mapleThread = new MapleCommandThread();
      mapleThread.maple = maple;
      mapleThread.cmd = mapleCmd;
      mapleThread.out = webOutput;
      mapleThread.parent = this;
      MapleFinished = false;

      // start up the thread      
      mapleThread.start();

      // now try to join with that thread 
      // if it finishes before the time limit it will wake us up 
      // otherwise we wake up when the time limit is reached.
      try {
         mapleThread.join(timeout);
      } catch (InterruptedException ie) {  
         logentry.close("ERROR");
         maple.condemned = true;
         maplePool.discardMaple(maple);

         msg_ = new StringWriter();
         msg = new PrintWriter(msg_);

         msg.write("Error while executing Maple command:\n" + 
                 mapleCmd + "-----------------\n" +
                "Exception message:\n" + ie.getMessage() + "\n\n" +
                "Stack trace:\n");

         ie.printStackTrace(msg);
         alicereq.deleteFiles();
      
         throw(new ServletException(msg_.toString()));
      }

      // No matter what happened, if the thread is still alive we are
      // in trouble. Hopefully this will relinquish any lock on the Maple object
      if (mapleThread.isAlive()) { 
         mapleThread.interrupt();
         mapleThread = null;
      }

      // if the thread did not finish, try to determine if it
      // timed out, or if it was destroyed.  Then kill the Maple process, 
      // stop the thread, and report what happened
      if (!MapleFinished) {
         // If the process was manually destroyed from the 
         // monitor page while processing our request
         // the maple object will already be discarded
         if (maple.discarded) {
            logentryMsg = "TERMINATED";
            webOutput.flush();
            webOutput_.flush();
            webOutput.println("<html><head>");
            webOutput.println("<title>Maple Terminated Message</title>");
            webOutput.println("<body><font color=\"#FF0000\">ERROR:</font> Your Maple process was terminated before it was able to complete your request.  This is probably not due to an error on your part.  Press the back button in your browser and try again.<br>\n");
            webOutput.println("</body></html>");
         }
         // or perhaps the execution of the maple command 
         // threw an exception which was stored in its exception field
         else if (maple.exception != null) {
            Exception e = maple.exception;
            logentry.close("ERROR");
            maple.condemned = true;
            maplePool.returnMaple(maple);

            msg_ = new StringWriter();
            msg = new PrintWriter(msg_);
            msg.write("Error while executing Maple command:\n" + 
                      mapleCmd + "-----------------\n" +
                      "Exception message:\n" + e.getMessage() + "\n\n" +
                      "Stack trace:\n");

            e.printStackTrace(msg);
            alicereq.deleteFiles();
      
            throw(new ServletException(msg_.toString()));
         }
         // Otherwise we should discard it and report that the process timed out
         else {

            maplePool.discardMaple(maple);

            logentryMsg = "TIMEOUT";
            // discard any partial output so that a broken web page/form
            // isn't sent to a user who tries to submit it.
            webOutput.flush();
            webOutput_.flush();
            // and report what went wrong and what request caused it
            webOutput.println("<html><head>");
            webOutput.println("<title>Maple Timeout Message</title>");
            webOutput.println("<body><font color=\"#FF0000\">ERROR:</font> The Maple process exceeded the time limit allowed for your request.  This can be caused by submitting an answer to a question that required too much time for Maple to process. Press the back button on your browser and check the values on the last page you submitted.  Contact the administrator or your instructor with the information below if you cannot fix the problem yourself.<br>");
            webOutput.println("<br>Time limit: " + timeout + " milliseconds");
            Enumeration prms = alicereq.getParameterNames();
            while (prms.hasMoreElements()) {
               String name = (String)prms.nextElement();
               if (name.indexOf("Password") == -1) {
                  webOutput.println("<br>" + name + ": " + alicereq.getParameter(name));
               }
            }
            webOutput.println("</body></html>");
         }
      }

      alicereq.deleteFiles();
      
/* dist
    } catch(Exception e) {
      logentry.close("ERROR");
      maple.condemned = true;
      maplePool.returnMaple(maple);

      msg_ = new StringWriter();
      msg = new PrintWriter(msg_);

      msg.write("Error while executing Maple command:\n" + 
                "`aim/Main`(" + commandargs + "):\n-----------------\n" +
                "Exception message:\n" + e.getMessage() + "\n\n" +
                "Stack trace:\n");

      e.printStackTrace(msg);
      
      throw(new ServletException(msg_.toString()));
    } finally {
      alicereq.deleteFiles();
    }
end dist */

    page = webOutput_.toString();
    boolean complete = true;

    if (page.startsWith("ContentType=")) {
      int i = "ContentType=".length();
      int j = page.indexOf("\n");
      if (j != -1) {
        response.setContentType(page.substring(i,j));
        page = page.substring(j+1);
      } else {
        response.setContentType("text/html");   
      } 
    } else {
      if (page.indexOf("</html>") == -1  && logentryMsg == null) {
// KM
// dist        complete = false;
        logentryMsg = "INCOMPLETE";
// END KM
      }
      response.setContentType("text/html");
    }

    response.getWriter().print(page);
    response.getWriter().close();

// KM
    if (logentryMsg == null) {
      logentry.close();
    } else {
      logentry.close(logentryMsg);
    }
// END KM

    maplePool.returnMaple(maple);
    maplePool.clean(1000000,10000000);
  }

  public void printBusyPage(HttpServletResponse response) {
    try {
      response.setContentType("text/html");
      PrintWriter webOutput = response.getWriter();
      webOutput.println("<html><head></head><body bgcolor='white'>");
      webOutput.println(BusyMessage);
      webOutput.println("</body></html>");
    } catch(IOException e) {}
  }

  public void logRequest(AliceRequest alicereq) 
    throws IOException,ServletException {
    Enumeration names;
    String name;
    StringWriter rlog;
    String[] values;
    String value;

    if (LogFile != null) {
      rlog = new StringWriter();
      rlog.write("Request parameters:\n");
      names = alicereq.getParameterNames();
      while(names.hasMoreElements()) {
        name = (String)names.nextElement();
        values = alicereq.getParameterValues(name);
        
        rlog.write(name + ": " + values.length + " value(s)\n");
        for (int i = 0; i < values.length; i++) {
          value = values[i];
          rlog.write("> " + Maple.quote(value) + " (" + value.length() + ")\n");
        }
      }    

      rlog.write("\n\n");
      writeLog(rlog.toString());
    }
  }

  private void doServletCommand(String command,
                                HttpServletRequest request,
                                HttpServletResponse response)
    throws ServletException, IOException
  {

// KM
    // this provides a hook for debugging the servlet.  The idea is that you
    // insert whatever java code here that you want to test
    // then recompile, fire up the server, and call the URL
    // http://localhost/Alice?Command=ServletDebug and it will run
    // this code for you
    if (command.equals("ServletDebug")) {

      Maple maple = maplePool.getMaple("adminKen");
      response.setContentType("text/html");
      PrintWriter out = response.getWriter();
      out.println("<html>");
      out.println("<head>");
      out.println("<title>Alice Servlet Debugger</title>");
      out.println("<body>");
      out.println("<h1><font color=\"#0000FF\">Alice Servlet Debugger</font></h1>");

      // Insert test code here

      try {
        maple.exec("print(\"Hello World\");",out);
        out.println("<br><br>");
      } catch (Exception ex) {
        out.println("Exception encountered:\n<br>\n" + ex.getMessage() + "\n<br>\n");
        ex.printStackTrace(out);
      }

      out.println("<br><br><font color=\"#0000FF\">Thank you for using the Alice debugger!</font>");
      out.println("</body></html>");

    }
    // otherwise behave normally
    else {
// END KM

      if (command.equals("ServletClose")) {
        maplePool.drain();
      } else if (command.equals ("ServletDestroy")) {
        maplePool.scorch();
// KM
        aliceLog.scorch();
// END KM
      }

      response.setContentType("text/html");
      PrintWriter out = response.getWriter();
      out.println("<html>");
      out.println("<head>");
      out.println("<title>Alice servlet monitor</title>");
      out.println("<script language=\"javascript\">");
      out.println("<!--");
      out.println("function f(c) {");
      out.println(" document.aimform.Command.value = 'Servlet' + c;");
      out.println(" document.aimform.submit();");
      out.println("};");
      out.println("// -->");
      out.println("</script>");
      out.println("<body bgcolor='white'>");
      out.println("<h1>Alice server monitor</h1>");
      DateFormat d = DateFormat.getTimeInstance(DateFormat.SHORT);
      out.println("Page last refreshed " + d.format(new Date()));
      out.println("<hr noshade>");
      out.println("<table width='100%'><tr>");
      out.println("<td><a href=\"javascript:f('Monitor');\">Refresh</a></td>");
      out.println("<td><a href=\"javascript:f('Close');\">Close</a>");
      out.println(" all processes</td>");
      out.println("<td><a href=\"javascript:f('Destroy');\">Destroy</a>");
      out.println(" all processes</td>");
      out.println("</tr></table>");
      out.println("<hr noshade>");
      out.println("<form name='aimform' action='Alice' method='POST'>");
      out.println("<input type='hidden' name='Command' value='ServletMonitor'>");
      out.println(maplePool.statusReport());
      out.println("<hr noshade>");
      out.println("<h2>Recent requests</h2>");
      out.println(aliceLog.toHTML());
      out.println("<hr noshade>");
      out.println("<h2>Initialization parameters</h2>");
      out.println("<table border width='100%'>");
      out.println(initParamLine("AliceMapleDir",AliceMapleDir));
      out.println(initParamLine("TempDir",TempDir));
      out.println(initParamLine("MaxPostSize","" + MaxPostSize));
      out.println(initParamLine("MapleProgram",MapleProgram));
      out.println(initParamLine("MapleInitOptions",MapleInitOptions));
      out.println(initParamLine("MaxMapleProcesses","" + MaxMapleProcesses));
      out.println(initParamLine("LogFile",LogFile));
      out.println(initParamLine("LastRequestFile",LastRequestFile));
      out.println(initParamLine("TtHProgram",TtHProgram));
      out.println(initParamLine("TtHLogFile",TtHLogFile));
      out.println(initParamLine("TtHLogLevel","" + TtHLogLevel));
// KM
      out.println(initParamLine("KillProgram",KillProgram));
      out.println(initParamLine("KillProgramArg",KillProgramArg));
      out.println(initParamLine("AliceHistorySize","" + AliceHistorySize));
      out.println(initParamLine("StudentTimeLimit","" + StudentTimeLimit));
      out.println(initParamLine("AdminTimeLimit","" + AdminTimeLimit));
// END KM
      out.println("</table>");
      out.println("<hr noshade>");
      out.println("<table width='100%'><tr>");
      out.println("<td><a href=\"javascript:f('Monitor');\">Refresh</a></td>");
      out.println("<td><a href=\"javascript:f('Close');\">Close</a>");
      out.println(" all processes</td>");
      out.println("<td><a href=\"javascript:f('Destroy');\">Destroy</a>");
      out.println(" all processes</td>");
      out.println("</tr></table>");
      out.println("<hr noshade>");
      out.println("</form>");
      out.println("</body></html>"); 

// KM
    }
// END KM

  }

  private String initParamLine(String n,String p) {
    String s;
    if (p == null) {
      return("<tr>\n<td>" + n + "</td>\n<td>(missing)</td>\n</tr>\n");
    } else {
      return("<tr>\n<td>" + n + "</td>\n<td>" + p + "</td>\n</tr>\n");
    }
  }

  private void writeLog(String s) 
    throws ServletException {
    if (LogFile != null) {
      try {
        PrintWriter logWriter = new PrintWriter(new FileWriter(LogFile,true));
        DateFormat df = DateFormat.getDateTimeInstance();
        logWriter.println(df.format(new Date()));
        logWriter.println(s);
        logWriter.close();
      } catch (Exception e) {
        throw new ServletException("Could not write log file\n",e);
      }
    }
  }

  private String ParamReport() {
    StringBuffer s = new StringBuffer();
    s.append("Alice initialization parameters:\n");

    s.append("AliceMapleDir=" + AliceMapleDir + "\n");
    if (MapleProgram == null) {
      s.append("MapleProgram=null\n");
    } else {
      s.append("MapleProgram=" + MapleProgram + "\n");
    }

    if (MapleInitOptions == null) {
      s.append("MapleInitOptions=null\n");
    } else {
      s.append("MapleInitOptions=" + MapleInitOptions + "\n");
    }

    if (LogFile == null) {
      s.append("LogFile=null\n");
    } else {
      s.append("LogFile=" + LogFile + "\n");
    }

    if (LastRequestFile == null) {
      s.append("LastRequestFile=null\n");
    } else {
      s.append("LastRequestFile=" + LastRequestFile + "\n");
    }

    if (TtHProgram == null) {
      s.append("TtHProgram=null\n");
    } else {
      s.append("TtHProgram=" + TtHProgram + "\n");
    }

    if (TtHLogFile == null) {
      s.append("TtHLogFile=null\n");
    } else {
      s.append("TtHLogFile=" + TtHLogFile + "\n");
    }

    return( s.toString() );
  }

  private void initMaple(Maple maple) throws ServletException {
    
    // This is for trapping output from Maple commands that are not 
    // expected to produce any output.  If they do produce any, we
    // need to log it in an error message.
    StringWriter trapOutput_,msg_;
    PrintWriter trapOutput,msg;
    String initcommand;

    try { 
      trapOutput_ = new StringWriter();
      trapOutput = new PrintWriter(trapOutput_);
      initcommand = "currentdir(\"" + AliceMapleDir + "\"):\n";
      maple.exec(initcommand,trapOutput);
    } catch (Exception ex) {
      //TODO: check that these \n's are OK in Windows
      msg_ = new StringWriter();
      msg = new PrintWriter(msg_);
      msg.write("Exception while starting new Maple process:\n" +
                ex.getMessage() + "\n");
      ex.printStackTrace(msg);

      if (maple != null) {
        maplePool.discardMaple(maple);
      }
      throw(new ServletException(msg_.toString()));
    }

    if (trapOutput_.toString().trim().length() > 0) {
      String err = "Unexpected output while starting new " +
        "Maple process:\n" + trapOutput_.toString() + "\n";
      maplePool.discardMaple(maple);
      throw(new ServletException(err));
    }

    // Load the initialization file.  Note that this is just a short
    // text file, which loads the main main part of the software by
    // evaluating the command "With(Aim)".  Maple knows that it can
    // ignore this if Aim has already been loaded, so reading
    // AliceServer.mpl should be very fast after the first time.
    
    // This initialization should not produce any output.  If it does,
    // we give up in disgust.
    trapOutput_ = new StringWriter();
    trapOutput = new PrintWriter(trapOutput_);
    
    try {
      maple.exec("read \"AliceServer.mpl\":", trapOutput);
    } catch(Exception ex) {
      maplePool.discardMaple(maple);

      msg_ = new StringWriter();
      msg = new PrintWriter(msg_);
      msg.write("Error while reading AliceServer.mpl:\n" + 
                ex.getMessage() + "\n");
      ex.printStackTrace(msg);
      throw(new ServletException(msg_.toString()));
    }

    if (trapOutput_.toString().trim().length() > 0) {
      maplePool.discardMaple(maple);

      String err = "Unexpected output while initializing new " +
        "Aim process:\n" + trapOutput_.toString() + "\n";
      throw(new ServletException(err));
    }

  }


  // This will be called when the web server shuts down.  
  public void destroy() {
    // close all Maple processes
    try {
      writeLog("Closing down servlet\n");
      maplePool.drain();
    } catch (Exception ex) {}
  }

//KM
  public int timeLimit(String cmd) {
   
    if (cmd == null               ||
        cmd.startsWith("admin")   ||
        cmd.startsWith("analyze") ||
        cmd.startsWith("try")     ||
        cmd.startsWith("review")) {
        return(AdminTimeLimit); 
    } else {
        return(StudentTimeLimit);
    }
  }
// END KM

}

// KM
// In order to timeout errant Maple commands, we use a separate thread 
// to execute a Maple command and destroy the process if we can't join up 
// with the thread in the specified timout period.  This class represents
// that thread.
/******************************************************************
  Create a class for running Maple processes
******************************************************************/
class MapleCommandThread extends Thread {
   
   String cmd = "";
   Maple maple;
   PrintWriter out;
   Alice parent;   

   public void run() {
     try {
       maple.exec(cmd,out);
       if (!maple.discarded) {
         parent.MapleFinished = true;
       }
     } catch (Exception e) { 
       maple.exception = e;
     } 
   }
}
// END KM


/**
 * A utility class encoding a request that might or might not be
 * of content type multipart/form-data.  Typically requests that
 * include a file upload will have this type, whereas requests
 * without file uploads will have type text/plain (?)
 */
/******************************************************************
  Create a class to represent a request.
******************************************************************/
class AliceRequest {
  private HttpServletRequest request;
  private MultipartRequest mprequest;
  private Vector parameterNames;
  public  String studentID;
  public  String command;
  public  String subject;
  public  String quiz;

  static final String[] specialNames =
    new String[] {
      "StudentID","Password","AdminPassword","Command","Action",
      "SubjectName","QuizName","Seed"
    };

  public AliceRequest(HttpServletRequest r,
                      String uploadDirectory,
                      int maxPostSize) throws IOException {
    int i;
    Enumeration names;
    String name,value;
    
    request = r;
    try {
      mprequest =
        new MultipartRequest(r,
                             uploadDirectory,
                             maxPostSize);
    } catch(IOException e) {
      if (e.getMessage() ==
          "Posted content type isn't multipart/form-data") {
        mprequest = null;
      } else {
        e.fillInStackTrace();
        throw(e);
      }
    }

    // Now set parameterNames to be the list of parameter names as 
    // returned by getParameterNames(), but sorted into a more
    // convenient order.
    parameterNames = new Vector();
    for (i = 0; i < specialNames.length; i++) {
      if (mprequest == null) {
        value = request.getParameter(specialNames[i]);
      } else {
        value = mprequest.getParameter(specialNames[i]);
      }
      if (value != null) {
        parameterNames.addElement(specialNames[i]);
      }
    }

    if (mprequest == null) {
      names = request.getParameterNames();
    } else {
      names = mprequest.getParameterNames();
    }

    while (names.hasMoreElements()) {
      name = (String)names.nextElement();
      if (name.indexOf("RawAnswer") == -1 &&
          ! isSpecial(name)) {
        parameterNames.addElement(name);
      }
    }

    if (mprequest == null) {
      names = request.getParameterNames();
    } else {
      names = mprequest.getParameterNames();
    }

    while (names.hasMoreElements()) {
      name = (String)names.nextElement();
      if (name.indexOf("RawAnswer") != -1)
        parameterNames.addElement(name);
    }

    if (mprequest == null) {
      studentID   = r.getParameter("StudentID");
      command     = r.getParameter("Command");
      subject     = r.getParameter("SubjectName");
      quiz        = r.getParameter("quizName");
    } else {
      studentID   = mprequest.getParameter("StudentID");
      command     = mprequest.getParameter("Command");
      subject     = mprequest.getParameter("SubjectName");
      quiz        = mprequest.getParameter("quizName");
    }

    if ( command != null && command.indexOf("/") != -1) {
      studentID = "(admin)";
    }

    if (studentID == null) { studentID = ""; }
    if (command == null)   { command = ""; }
    if (subject == null)   { subject = ""; }
    if (quiz == null)      { quiz = ""; }
  }

  private static boolean isSpecial(String p) {
    int i;
    for (i = 0; i < specialNames.length; i++) {
      if (p.equals(specialNames[i])) {
        return(true);
      }
    }
    return(false);
  }

  public boolean isMultiPart() {
    return (mprequest != null);      
  }

  public Enumeration getParameterNames() {
    return parameterNames.elements();
  }

  public Enumeration getFileNames() {
    if (mprequest == null) {
      return((new Vector()).elements());
    } else {
      return mprequest.getFileNames();
    }
  }

  public String getParameter(String name) {
    if (mprequest == null) {
      return(request.getParameter(name));
    } else {
      return(mprequest.getParameter(name));
    }
  }

  public String[] getParameterValues(String name) {
    if (mprequest == null) {
      return(request.getParameterValues(name));
    } else {
      return(mprequest.getParameterValues(name));
    }
  }

  public String getFilesystemName(String name) {
    if (mprequest == null) {
      return(null);
    } else {
      return(mprequest.getFilesystemName(name));
    }
  }

  public String getOriginalFileName(String name) {
    if (mprequest == null) {
      return(null);
    } else {
      return(mprequest.getOriginalFileName(name));
    }
  }

  public String getContentType(String name) {
    if (mprequest == null) {
      return(null);
    } else {
      return(mprequest.getContentType(name));
    }
  }

  public File getFile(String name) {
    if (mprequest == null) {
      return(null);
    } else {
      return(mprequest.getFile(name));
    }
  }
  
  public void deleteFiles() {
    Enumeration files = getFileNames();
    String name;
    File file;

    while(files.hasMoreElements()) {
      name = (String)files.nextElement();
      file = getFile(name);
      try {
        file.delete();
      } catch (Exception e) {;}
    }
  }

  public String mapleCommandArgs() {
    StringWriter params = new StringWriter();    
                     
    Enumeration names = parameterNames.elements();
    String name,value;
    File serverfile;
    String serverfilename,originalfilename;

    while(names.hasMoreElements()) {
      name = (String)names.nextElement();
      value = getParameter(name);
        
      params.write(",\"" + name + "\"=");
      params.write(Maple.quote(value));
    }
    
    names = getFileNames();

    while(names.hasMoreElements()) {
      name = (String)names.nextElement();
      if ( getFile(name) != null &&
           (serverfilename = getFile(name).getAbsolutePath()) != null) {
        params.write(",\"" + name + "\"=" +
                     Maple.quote(serverfilename));
      }
      originalfilename = getOriginalFileName(name);
      if (originalfilename != null) {
        params.write(",\"" + name + "Original\"=" +
                     Maple.quote(originalfilename));
      }
    }
    params.flush();
    
    String paramstring = params.toString();
    if (paramstring.startsWith(",")) {
      paramstring = paramstring.substring(1);
    }

    return paramstring;
  }
}

/******************************************************************
  Create a class to represent a server log entry
******************************************************************/
class AliceLogEntry {
  public int processID;

  public Date startTime,endTime;
  public String command;
  public boolean isStudent;
  public String studentID;
  public String subject;
  public String quiz;
  public String status;

  public AliceLogEntry (AliceRequest a) {
    studentID = (a.studentID.length() == 0) ? "&nbsp;" : a.studentID;
    command   = (a.command.length()   == 0) ? "&nbsp;" : a.command;
    subject   = (a.subject.length()   == 0) ? "&nbsp;" : a.subject;
    quiz      = (a.quiz.length()      == 0) ? "&nbsp;" : a.quiz;
    processID = -1;
    startTime = new Date();
    endTime = null;
    status = "OK";
  }

  public void close() {
    endTime = new Date();
    status = "DONE";
  }

  public void close(String s) {
    endTime = new Date();
    status = "<font color='red'>" + s + "</font>";
  }

  public String toHTML() {
    String id,st,tt;
    DateFormat df = DateFormat.getTimeInstance(DateFormat.SHORT);

    if (processID < 0) {
      id = "&nbsp;";
    } else {
      id = "" + processID;
    }

    if (startTime == null) {
      st = "&nbsp;";
      tt = "&nbsp;";
    } else {
      st = df.format(startTime);
      if (endTime == null) {
        tt = "" + (new Date().getTime() - startTime.getTime()) + "+";
      } else {
        tt = "" + (endTime.getTime()  - startTime.getTime());
      }
    }

    return("<tr>\n" + 
           "<td>" + id + "</td>\n" +
           "<td>" + st + "</td>\n" +
           "<td>" + tt + "</td>\n" + 
           "<td>" + studentID + "</td>\n" + 
           "<td>" + command + "</td>\n" +
           "<td>" + subject + "</td>\n" +
           "<td>" + quiz + "</td>\n" +
           "<td>" + status + "</td>\n" +
           "</tr>\n");
  }
}

/******************************************************************
  Create a class which will represent the server log object 
******************************************************************/
class AliceLog {
  private Vector entries;
  private int size;
  
  public AliceLog(int size) {
    this.entries = new Vector(size);
    this.size = size;
  }

  public void addEntry(AliceLogEntry e) {
    synchronized (this) {
      while (entries.size() >= size) {
        entries.removeElementAt(0);
      }
      entries.addElement(e);
    }
  }

// KM
  public void scorch() {
    String s;
    Enumeration ents;
    AliceLogEntry e;

    s = "TERMINATED";
    synchronized(this) {
      ents = entries.elements();
      while(ents.hasMoreElements()) {
        e = (AliceLogEntry)ents.nextElement();
        if (e.status.equals("OK")) {
          e.close("TERMINATED");
        }
      }
    }

    
  }
// END KM

  public String toHTML() {
    String s;
    Enumeration ents;
    AliceLogEntry e;

    s = "";

    synchronized(this) {
      ents = entries.elements();
      while(ents.hasMoreElements()) {
        e = (AliceLogEntry)ents.nextElement();
        s = e.toHTML() + s;
      }
    }

    s =
      "<table border width='100%'>\n" +
      "<tr>\n" + 
      "<th>Process</th>" +
      "<th>Start time</th>" + 
      "<th>Time taken</th>" +
      "<th>Student ID</th>" +
      "<th>Command</th>" +
      "<th>Subject</th>" +
      "<th>Quiz</th>" +
      "<th>Status</th>" +
      "</tr>" + 
      s +
      "</table>\n";

    return(s);
  }
}

// Local Variables:
// c-basic-offset: 2
// compile-command: "javac -d /aim/WEB-INF/classes Alice.java"
// End:

