printf("AIM will now try to configure your system automatically.\n"):

read("Util0.mpl"):
read("OS0.mpl"):

Config := table([]):
unassign('`Date/LocalTimeZoneName`'):
unassign('`Date/LocalTimeZoneOffset`'):

traperror(fclose("autoconf.log")):
traperror(fremove("autoconf.log")):

logd := traperror(fopen("autoconf.log",'WRITE')):

if logd = lasterror then
 printf("Could not open autoconf.log for writing - no log will be written\n\n");
 p := proc() printf(args); end:
else
 printf("All messages from the configuration process will also be written\n"):
 printf("in the file autoconf.log\n\n"):
 p := proc() printf(args); fprintf(logd,args); end:
fi:

if (waffle = true) then
 q := eval(p):
else
 q := proc() 1; end:
fi:

setconfig :=
 proc(k::name,v::{integer,string,boolean})
  # Set Config[k] to v unless Config[k] is already defined and
  # has the same type as v or is NULL.
  global Config;

  if member([k],[indices(Config)]) then
   if not(type([Config[k]],[whattype(v)])) and Config[k] <> NULL then
    q("Config[%A] was set manually to %A\n",k,Config[k]);
    q("This has the wrong type and is being overridden by the default,\n"):
    q("which is %A.\n\n",v):
    Config[k] := v:
   fi:
  else
   Config[k] := v:
  fi:
 end:

######################################################################

# Determine operating system and Maple version number

printf("%s%s\n",os_vnummsg,os_osmsg):

if (os_vnum = 0) then stop; fi:

Config['MapleVersion']    := os_vnum:
Config['OperatingSystem'] := os_os:

######################################################################

# Define some functions to access the file system

strip := proc(s::string,t::string)
 if length(s) >= length(t) and
    substring(s,-length(t) .. -1) = t then
  RETURN(substring(s,1..(-length(t)-1)));
 else
  RETURN(NULL):
 fi:
end:


######################################################################

# Read any customized definitions

if os_fileexists("ManualConfig.mpl") then
 p("Reading user supplied options from ManualConfig.mpl\n\n"):
 e := interface(echo):
 interface(echo) := 2:
 read("ManualConfig.mpl"):
 interface(echo) := e:
fi:

######################################################################


path := getenv("PATH"):
if path = NULL then
 path := []: 
else 
 path := [os_pathsplit(path)]:
fi:

look_for_dir :=
 proc(target::string)
  local msg,checks,places,ans,newdirs,l,r,c,d,
        place,x,y,m,prog,progdir,ok,found,location;
  global problems;

  p("\nLooking for %s\n",target):

  found := false:
  location := NULL:

  checks,places :=
   selectremove(e -> evalb(lhs(e) = "Check"),[args[2..-1]]):
  
  checks := map(rhs,checks):

  ans := NULL;
  newdirs := []:

  for place in places do
   l := lhs(place):
   r := rhs(place):
   newdirs := NULL:
   if l = "Conf" then
    if member([r],[indices(Config)]) then
     d := Config[r];
     if type([d],[string]) then
      newdirs := [d];
      q("Config['%a'] was set manually to %a\n",r,d):
     else
      q("Config['%a'] was set to %A,\n which is not a string\n",
        r,d):
     fi:
    else
     q("Config['%a'] is not set\n",r):
    fi:
   elif l = "Env" then
    newdirs := [getenv(r)]:
    if newdirs = [] then
     q("Environment variable %s is not set\n",r):
    else
     q("Environment variable %s is set to %s\n",r,op(newdirs)):
    fi:
   elif l = "Def" then
    d := r[1]:
    q("Checking default location %s/%s* ...",d,r[2]):
    if r[1] = "" or r[1] = "c:" or r[1] = "C:" then
     newdirs := os_listfiles(cat(d,"/")):
    else 
     newdirs := os_listfiles(d):
    fi:
    newdirs := select(util_startswith,newdirs,r[2]):
    newdirs := map((x,y) -> os_osname(cat(y,"/",x)),newdirs,d):
    newdirs := select(os_direxists,newdirs):
    if newdirs = [] then
     q("not found\n"):
    fi:
   elif l = "Path" then
    q("Checking the path for executables like */%s ...",r):
    progdir,prog := op(util_splitfilename(r)):
    if os_os = "Linux" then
     d := util_striptrailspaces(os_system0("which %a",prog)):
     if util_endswith(d,r) then
      d := substring(d,1..-length(r)-1):
      newdirs := [d]:
     fi:
    else
     newdirs := select(util_endswith,path,progdir):
     newdirs :=
      select((x,y) -> os_fileexists(cat(x,"/",y)),newdirs,prog):
     newdirs := map((x,m) -> substring(x,1..-m-1),newdirs,length(r)):
    fi:
    if newdirs = [] then
     q("not found\n"):
    fi:
   fi:

   for d in newdirs do
    q("trying %s ... ",d):
    if os_direxists(d) then
     q("exists ... "):
     ok := true:
     for c in checks do
      if not(os_fileexists(cat(d,"/",c))) then
       ok := false:
       break:
      fi:
     od:
     if ( ok ) then
      q(" OK\n"):
      p("%s found at %s\n",target,d):
      found := true:
      location := d:
      if (testmode <> true) then
       RETURN(d):
      fi:
     else
      q(" not OK (does not contain %s)\n",c):
     fi:
    fi:
   od:
  od:
  if (found <> true) then
   p("Could not find %s\n",target):
   problems := true:
  fi:
  RETURN(location):
 end:
 
##################################################

look_for_prog :=
 proc(target::string)
  local msg,checks,places,ans,l,r,c,d,place,x,y,m,f,ok,e,cmd,location;
  global problems;

  p("\nLooking for %s\n",target):
  location := NULL:

  checks,places :=
   selectremove(e -> evalb(lhs(e) = "Check"),[args[2..-1]]):
  
  checks := map(rhs,checks):

  ans := NULL;

  for place in places do
   l := lhs(place):
   r := rhs(place):
   f := NULL:
   if l = "Conf" then
    if member([r],[indices(Config)]) then
     d := Config[r];
     if type([d],[string]) then
      f := d;
      q("Config['%a'] was set manually to %a\n",r,d):
     else
      q("Config['%a'] was set to %A,\n which is not a string\n",
        r,d):
     fi:
    else
     q("Config['%a'] is not set\n",r):
    fi:
   elif l = "Def" then
    f := r:
    if os_fileexists(f) then
     q("Found at default location %s\n",f)
    else
     q("Nothing in default location %s\n",f):
     f := NULL;
    fi:
   elif l = "Path" then
    q("Checking the path for executables like */%s ...",r):
    if os_os = "Linux" then
     d := util_striptrailspaces(os_system0("which %a",r)):
     if os_fileexists(d) then
      f := d: 
      q("found at %s\n",f):
     else
      q("not found\n"):
     fi:
    else
     d := map((x,y) -> cat(x,"/",y),path,r): 
     d := select(os_fileexists,d):
     if d <> [] then 
      f := d[1]: 
      q("found at %s\n",f):
     else
      q("not found\n"):
      f := NULL;
     fi:
    fi:
   fi:

   if f <> NULL then
    f := os_osname(f):
    q("Trying %s ... ",f):
    if os_fileexists(f) then
     q("exists ... "):
     ok := true:
     for c in checks do
      cmd := cat("\"",f,"\" ",c[1]);
      e := traperror(os_system(cmd)):
      if e = lasterror then
       q("\nNot OK : error while executing %a : %s\n",cmd,e):
      elif SearchText(c[2],e) = 0 then
       q("\nNot OK : '%s %s' does not produce '%s' in the output.\n",
         f,c[1],c[2]):
       if length(e) > 300 then e := cat(substring(e,1..300),"..."): fi:
       q("Output was: \n%s\n\n",e):
       ok := false;
       break:
      fi:
     od:
     if ( ok ) then
      q(" OK\n"):
      p("%s found at %s\n",target,f):
      location := f:
      if (testmode <> true) then RETURN(f): fi:
     else
      q("Not OK\n"):
     fi:
    else
     q("does not exist.\n"):
    fi:
   fi:
  od:
  if (location = NULL) then
   p("Could not find %s\n",target):
   problems := true:
  fi:
  RETURN(location):
 end:

problems := false;

######################################################################

# Look for Java

# Config['JavaHome']
# JAVA_HOME in the environment
# Defaults: Linux: /usr/java/jdk*, /usr/local/jdk*
#           DOS: c:\java\jdk*, c:\jdk*
# double parent of java executable (try 'where', or PATH)
#
# Check: should contain file bin/java

if os_os = "Linux" then
 Config['JavaHome'] :=
 look_for_dir(
  "Java",
  "Conf"  = 'JavaHome',
  "Env"   = "JAVA_HOME",
  "Def"   = ["/usr/java","jdk"],
  "Def"   = ["/usr/local","jdk"],
  "Path"  = "bin/java",
  "Check" = "bin/java"
 ):
else
 Config['JavaHome'] :=
 os_osname(look_for_dir(
  "Java",
  "Conf"  = 'JavaHome',
  "Env"   = "JAVA_HOME",
  "Def"   = ["C:\\java","jdk"],
  "Def"   = ["C:","jdk"],
  "Def"   = ["C:\\Program Files\\java","jdk"],
  "Def"   = ["C:\\Program Files","jdk"],
  "Path"  = "bin/java.exe",
  "Check" = "bin/java.exe"
 )):
fi:
 

######################################################################

# Look for Tomcat

cd := [util_slashsplit(currentdir())]:
if nops(cd) > 4 then
 cd := util_slashjoin(op(cd[1..-6])):
 cd := ("Def" = [cd,"tomcat"], "Def" = [cd,"jakarta-tomcat"]):
else
 cd := NULL:
fi:

if os_os = "Linux" then
 Config['CatalinaHome'] :=
 look_for_dir(
  "Tomcat",
  "Conf" = 'CatalinaHome',
  "Env"  = "CATALINA_HOME",
  "Def"  = ["/usr/local","tomcat"],
  "Def"  = ["/usr/local","jakarta-tomcat"],
  cd,
  "Check" = "bin/catalina.sh"
 ):
else
 Config['CatalinaHome'] :=
 os_osname(look_for_dir(
  "Tomcat",
  "Conf" = 'CatalinaHome',
  "Env"  = "CATALINA_HOME",
  "Def"  = ["C:","tomcat"],
  "Def"  = ["C:","jakarta-tomcat"],
  "Def"  = ["C:\\Program Files","tomcat"],
  "Def"  = ["C:\\Program Files","jakarta-tomcat"],
  cd,
  "Check" = "bin/catalina.bat"
 )):
fi:


######################################################################

# Look for Maple

if os_os = "Linux" then
 Config['MapleProgram'] :=
 look_for_prog(
  "Maple",
  "Conf"  = 'MapleProgram',
  "Def"   = "/bin/maple",
  "Def"   = "/usr/bin/maple",
  "Def"   = "/usr/local/bin/maple",
  "Def"   = "/usr/local/maple/bin/maple",
  "Def"   = "/usr/local/maple_su/bin/maple",
  "Path"  = "maple",
  "Check" = ["-c quit;","Waterloo"]
 ):
else
 Config['MapleProgram'] :=
 os_osname(look_for_prog(
  "Maple",
  "Conf"  = 'MapleProgram',
  "Def"   = "C:\\Maple\\bin.wnt\\cmaple.exe",
  "Def"   = "C:\\Maple 6\\bin.wnt\\cmaple.exe",
  "Def"   = "C:\\Maple 7\\bin.wnt\\cmaple.exe",
  "Def"   = "C:\\Maple 8\\bin.wnt\\cmaple.exe",
  "Def"   = "C:\\Program Files\\Maple\\bin.wnt\\cmaple.exe",
  "Def"   = "C:\\Program Files\\Maple 6\\bin.wnt\\cmaple.exe",
  "Def"   = "C:\\Program Files\\Maple 7\\bin.wnt\\cmaple.exe",
  "Def"   = "C:\\Program Files\\Maple 8\\bin.wnt\\cmaple.exe",
  "Path"  = "cmaple.exe",
  "Check" = ["-c quit;","Waterloo"]
 )):
fi:

######################################################################

# Look for TtH

if os_os = "Linux" then
 Config['TtHProgram'] :=
 look_for_prog(
  "TtH",
  "Conf"  = 'TtHProgram',
  "Path"  = "tth",
  "Def"   = "/usr/local/bin/tth",
  "Def"   = "/usr/bin/tth",
  "Def"   = "/bin/tth",
  "Check" = ["-r < tthtest.tex","TtH is OK"]
 ):
else
 Config['TtHProgram'] :=
 os_osname(look_for_prog(
  "TtH",
  "Conf"  = 'TtHProgram',
  "Path"  = "tth.exe",
  "Def"   = "C:\\TtH\\tth.exe",
  "Def"   = "C:\\Program Files\\tth.exe",
  "Def"   = "C:\\Program Files\\TtH\\tth.exe",
  "Check" = ["-r < tthtest.tex","TtH is OK"]
 )):
fi:

# KM
# dist p("\n"):
######################################################################

# Look for the Kill Program

if os_os = "Linux" then
 Config['KillProgram'] :="/bin/kill";
 os_osname(look_for_prog(
   "Kill Process Utility",
   "Conf"  = 'KillProgram',
   "Path"  = "kill",
   "Def"   = "/bin/kill",
   "Def"   = "/usr/bin/kill",
   "Def"   = "/usr/local/bin/kill",
   "Check" = ["","kill"]
  )):
 Config['KillProgramArg'] :="-9";
else
 Config['KillProgram'] :=
 os_osname(look_for_prog(
  "Kill Process Utility",
  "Conf"  = 'KillProgram',
  "Path"  = "Process.exe",
  "Def"   = "C:\\AIM\\Process",
  "Def"   = "C:\\Process",
  "Check" = ["-v","beyondlogic.org"]
 )):
 setconfig('KillProgramArg',"-k");
fi:

# END KM

######################################################################

# Look for email program

if os_os = "Linux" then
 Config['SendmailProgram'] :=
 look_for_prog(
  "sendmail",
  "Conf"  = 'SendmailProgram',
  "Path"  = "sendmail",
  "Def"   = "/usr/lib/sendmail",
  "Def"   = "/usr/bin/sendmail"
 ):
 setconfig('BLATProgram',""):
 setconfig('SMTPServer',""):
else
 Config['BLATProgram'] :=
 os_osname(look_for_prog(
  "BLAT",
  "Conf"  = 'BLATProgram',
  "Path"  = "blat.exe",
  "Def"   = "C:\\BLAT\\blat.exe",
  "Def"   = "C:\\Program Files\\blat.exe",
  "Def"   = "C:\\Program Files\\BLAT\\blat.exe"
 )):
 setconfig('SendmailProgram',"");
fi:

p("\n"):


######################################################################

# Look for upload directory

if os_os = "Linux" then
 Config['TempDir'] :=
 look_for_dir(
  "upload directory",
  "Conf"  = 'TempDir',
  "Env"   = "TempDir",
  "Env"   = "TMPDIR",
  "Def"   = ["","tmp"]
 ):
else
 Config['TempDir'] :=
 os_osname(look_for_dir(
  "upload directory",
  "Conf"  = 'TempDir',
  "Env"   = "TempDir",
  "Env"   = "TMPDIR",
  "Def"   = ["C:","temp"]
 )):
fi:

setconfig('MaxPostSize',10 * 1024 * 1024):

######################################################################

p("\nFinding the host name\n"):

if  type([Config['HostName']],[string]) then
 q("The host name was set manually to %a\n",Config['HostName']):
else
 if os_os = "Linux" then
  hostname := traperror(ssystem("hostname --fqdn")):
  if hostname = lasterror then
   err := hostname:
  elif hostname[1] > 0 then
   err := hostname[2]:
  else
   hostname := hostname[2]:
   err := NULL:
  fi:
  if err = NULL then
   if length(hostname) > 0 and substring(hostname,-1..-1)= "\n" then
    hostname := substring(hostname,1..-2):
   fi:
   q("'hostname --fqdn' returned %a\n",hostname):
   if (hostname = "localhost.localdomain") then
    hostname := "localhost":
   fi:
  else
   q("Error while running 'hostname --fqdn' :\n%s\n\n",err):
  fi:
 else
  hostname := "localhost":
 fi: 

Config['HostName'] := hostname:

fi:

p("Host name set to %s\n\n",Config['HostName']):

######################################################################

setconfig('Locale',"en"):

######################################################################

p("Setting time zone\n"):


timezone := NULL:
timeoffset := NULL:

zones :=
 table([
 "ADT" =-03,
 "AHST"=-10,
 "AST" =-04,
 "AT"  =-02,
 "BST" =+01,
 "BT"  =+03,
 "CAT" =-10,
 "CCT" =+08,
 "CDT" =-05,
 "CET" =+01,
 "CST" =-06,
 "EADT"=+11,
 "EAST"=+10,
 "EDT" =-04,
 "EET" =+02,
 "EST" =-05,
 "FST" =+02,
 "FWT" =+01,
 "GMT" =  0,
 "GST" =+10,
 "HDT" =-09,
 "HST" =-10,
 "IDLE"=+12,
 "IDLW"=-12,
 "JST" =+09,
 "MDT" =-06,
 "MEST"=+02,
 "MET" =+01,
 "MEWT"=+01,
 "MST" =-07,
 "NT"  =-11,
 "NZDT"=+13,
 "NZST"=+12,
 "PDT" =-07,
 "PST" =-08,
 "SST" =+02,
 "SWT" =+01,
 "UTC" =  0,
 "WADT"=+08,
 "WAST"=+07,
 "WAT" =-01,
 "WET" =  0,
 "YDT" =-08,
 "YST" =-09
]):


if os_os = "Linux" then
 timezone := traperror(util_striptrailspaces(os_system("date +%%Z")));
 if timezone = lasterror then timezone := NULL: fi:
 t0 := traperror(parse(os_system("date +\"(%%H*60+%%M)\""))):
 t0 := traperror(parse(os_system("date -u +\"(%%H*60+%%M)\""))):
 if type([t0,t1],[integer,integer]) then
  if t0 > t1 + 13 * 60 then
   t0 := t0 - 24 * 60:
  fi:
  timeoffset := round((t0 - t1)/15) * 15 * 60: 
 fi:
fi:

if type([`Date/LocalTimeZoneName`],[string]) then
 timezone := `Date/LocalTimeZoneName`:
 q("Time zone was set manually to %a\n",timezone):
elif timezone <> NULL then
 q("Time zone was determined automatically as %a\n",timezone):
else
 q("Could not determine time zone, assuming GMT\n"):
 timezone := "GMT":
fi:


if type([`Date/LocalTimeZoneOffset`],[integer]) then
 timeoffset := `Date/LocalTimeZoneOffset`:
 q("The offset from UTC was set manually to %d\n",timeoffset):
 if timeoffset <> 0 and abs(timeoffset) < 24 then
  q("This appears to be in hours, but it needs to be in seconds.\n"):
  q("It will be converted automatically.\n"):
  timeoffset := timeoffset * 3600:
 else
  if not (type(integer,timeoffset/900)) then
   timeoffset := round(timeoffset / 900) * 900:
   q("This was rounded to %d\n",timeoffset):
  fi:
 fi:
elif type([zones[timezone]],[integer]) then
 timeoffset := zones[timezone] * 3600:
 q("The offset from UTC is %d hour(s)\n",zones[timezone]):
elif timeoffset <> NULL then
 timeoffset := round(timeoffset / 900) * 900:
 q("The offset from UTC was calculated as %A hour(s)\n",
   timeoffset/3600.):
else
 q("The offset from UTC could not be determined, and will be taken\n"):
 q("to be zero. \n"):
 timeoffset := 0:
fi:

`Date/LocalTimeZoneName` := timezone:
`Date/LocalTimeZoneOffset` := timeoffset:

p("Time zone set to %s (= UTC +%A hours)\n",timezone,timeoffset/3600):

######################################################################

p("Setting other configuration options\n"):

if type([Config['TomcatPort']],[integer]) then
 Config['TomcatPort'] := sprintf("%d",Config['TomcatPort']):
fi:

setconfig('TomcatPort' , "80"):

if (Config['TomcatPort'] = "80") then
 setconfig('HostPort' , Config['HostName']):
else
 setconfig('HostPort' , cat(Config['HostName'],":",Config['TomcatPort'])):
fi:

setconfig('TomcatContext' , "ROOT"):

if Config['TomcatContext'] = "ROOT" then
 setconfig('FullContext',Config['HostPort']):
 setconfig('WebAppDir',
  os_maplename(cat(Config['CatalinaHome'],"/webapps/ROOT"))):
else
 setconfig('FullContext',
  cat(Config['HostPort'],"/",Config['TomcatContext'])):
 setconfig('WebAppDir' ,
  os_maplename(
   cat(Config['CatalinaHome'],"/webapps/",Config['TomcatContext']))):
fi:

setconfig('WEBINFDir' ,
 cat(Config['WebAppDir'],"/WEB-INF")):

setconfig('AimDir'   , Config['WEBINFDir']):
setconfig('LogDir'   , cat(Config['AimDir'],"/logs")):
setconfig('RootDir'  , cat(Config['AimDir'],"/root")):

setconfig('ServletLogFile',cat(Config['LogDir'],"/servletlog")):
setconfig('LastRequestFile',cat(Config['LogDir'],"/lastrequest")):
setconfig('EmailLogFile',cat(Config['LogDir'],"/emaillog")):

setconfig('WebDir' , Config['WebAppDir']):
setconfig('DocDir' , cat(Config['WebAppDir'],"/doc")):

setconfig('ServletURL', cat("http://",Config['FullContext'],"/Alice")):
setconfig('DocumentURL' , cat("http://",Config['FullContext'])):

setconfig('ShowToolBars' , true):

setconfig('MapleInitOptions',"default"):

# KM

setconfig('AliceHistorySize',20):
setconfig('MaxMapleProcesses',8):
setconfig('StudentTimeLimit',60000):  # one minute
setconfig('AdminTimeLimit',1200000):  # twenty minutes

# END KM

######################################################################

read("ConfigFields.mpl"):

p("Writing Config.mpl and ServletConfig.mpl\n"):

if os_fileexists("Config.mpl") then
 p("Renaming Config.mpl as Config.bak\n\n"):
 err := traperror(os_rename("Config.mpl","Config.bak")):
 if (err = lasterror) then
  p("Failed: %s\n",err):
  `quit`(1):
 fi:
fi:

m := proc() fprintf(fd,args); q(args); end:

addhash :=
 proc(s::string) 
  convert(subs(10 = op([10,35,32]),convert(s,bytes))[2..-1],bytes):
 end:

padeq :=
 proc(k::name,l::integer)
  m("'%a'%s = %a,\n",k,
    cat(" " $ (15 - length(k))),
    `if`(Config[k] = NULL,"NULL",Config[k])):
 end:

hdr := "
This file is generated automatically by AutoConf.mpl.  It is
recommended that you do not edit it directly.  Instead, you should
create a separate file called ManualConfig.mpl and copy over any
definitions (from either Config.mpl or ServletConfig.mpl) that you
wish to override.  You can then run AutoConf.mpl to fill in the
remaining definitions and check that everything is consistent.
":


fd := traperror(fopen("Config.mpl",'WRITE')):
if (fd = lasterror) then
 p("Could not open Config.mpl for writing:\n%a\n\n",fd):
 `quit`(1):
fi:

fprintf(fd,addhash(hdr)):
fprintf(fd,"\n\n"):

m("Config := table([\n"):

for x in ConfigFields do
 padeq(x[1],15):
 fprintf(fd,addhash(x[3])):
 fprintf(fd,"\n\n"):
od:

m("NULL]):\n\n"):

m("`Date/LocalTimeZoneName`   := %a:\n",`Date/LocalTimeZoneName`):
m("`Date/LocalTimeZoneOffset` := %d:\n",`Date/LocalTimeZoneOffset`):

m("\n\n"):

err := traperror(fclose(fd)):
if err = lasterror then
 p("Could not close Config.mpl:\n%a\n\n",err):
 `quit`(1):
fi:

fd := traperror(fopen("ServletConfig.mpl",'WRITE')):
if (fd = lasterror) then
 p("Could not open ServletConfig.mpl for writing:\n%a\n\n",fd):
 `quit`(1):
fi:

fprintf(fd,addhash(hdr)):
fprintf(fd,"\n\n"):

m("ServletConfig := table([\n"):

for x in ServletConfigFields do
 padeq(x[1],18):
 fprintf(fd,addhash(x[3])):
 fprintf(fd,"\n\n"):
od:

m("NULL]):\n\n"):

err := traperror(fclose(fd)):
if err = lasterror then
 p("Could not close ServletConfig.mpl:\n%a\n\n",err):
 `quit`(1):
fi:

######################################################################


if problems = true then
 p("There were configuration problems.\n"):
 p("You can read the above messages again by looking in the file  \n"):
 p("autoconf.log.                                                 \n"):
 if (waffle <> true) then
  p("You can get more detailed messages by setting waffle := true \n"):
  p("and reading in AutoConf.mpl again.                           \n"):
 fi:
 p("To fix the problems, edit ManualConfig.mpl and then run       \n"):
 p("AutoConf.mpl again.                                           \n"):
 if (dontquit <> true) then `quit`(1): fi:
else
 p("AIM was successfully configured.  You should now read in the  \n"):
 p("file Servlet.mpl to generate the servlet configuration files  \n"):
 p("and put them in their proper places.  Before doing this, you  \n"):
 p("may want to make a backup copy of the file                    \n"):
 p("<top tomcat directory>/conf/server.xml.                       \n"):
fi:



fclose(logd):
