interface(warnlevel = 0):
ConfigFields := [
 ['OperatingSystem',
  string,"
This is the operating system under which AIM is running.  The allowed
values are \"Linux\" and \"DOS\".  The Linux option has been tested
extensively under Red Hat Linux, and should be OK for most variants of
Unix, up to minor modification in the file OS0_Linux.mpl.  The DOS
option should be OK for all versions of Windows, but some functions
are missing, for example there is no facility to send email messages
or to determine accurately when a file was last modified.  (If you
know how to do these things, please send email to
N.P.Strickland@sheffield.ac.uk - I am a complete ignoramus about
anything to do with Microsoft.)"],

 ['MapleVersion',
  integer,"
The major version number of Maple under which AIM is running.  The
allowed values are 5, 6 or 7.  Maple 8 is not currently supported
because various features of the codegen package have been withdrawn
and replaced by a different mechanism.  (This will probably be a Good 
Thing in the long term.)"],

 ['ServletURL',
  string,"
The base URL used to access the Alice servlet."],

 ['DocumentURL',
  string,"
The base URL used to access static documents on the server."],

 ['WebDir',
  string,"
The top directory where static documents are stored on the server."],

 ['DocDir',
  string,"
The top directory where documentation files are stored on the server."],

 ['RootDir',
  string,"
The top of the directory heirarchy where AIM's data files (with 
information about subjects, quizzes, questions and students) are
stored."],

 ['ShowToolBars',
  boolean,"
true if the administrator's browser windows should have a full set of
toolbars; false if they should be omitted to save space."],
 
 ['Locale',
  string,"
The locale that determines the language of AIM's messages and so on.
There is a systematic framework for supporting languages other than
English, but so far no translation files have been prepared."],

 ['SMTPServer',
  string,"
The SMTP server used by BLAT for sending email under DOS/Windows.
Under Linux, AIM uses sendmail; I don't know the relationship
between this and SMTP."],

 ['BLATProgram',
  string,"
The absolute path to the BLAT program."],

 ['SendmailProgram',
  string,"
The absolute path to the sendmail program."],

 ['EmailLogFile',
  string,"
The log file for email messages sent out by AIM.  Currently this is
only used under Windows.  If it is NULL or the empty string, logging
is turned off."],

 ['TempDir',
  string,"
A directory for temporary storage of various things, including files 
uploaded by the servlet.  (The Maple part of the system will delete them
or move them to a more permanent home moments later.) Under Windows, 
the path should use backslashes as separators, and should not contain 
any spaces.  The default is /tmp (under Linux) or c:\\temp (under Windows)."]
]:

##################################################

ServletConfigFields := [
 ['JavaHome',
  string,"
The top directory for the Java distribution.  This is used to set the
environment variable JAVA_HOME in the script (startaim.sh or 
startaim.bat) that is used to start the server.  This variable must
be set to tell Tomcat where to find its basic java class files.  The
recommended location is /usr/java/jdk (under Linux) or c:\\jdk (under
Windows)."],
  
 ['CatalinaHome',
  string,"
The top directory for the Tomcat distribution.  This is used to set the
environment variable CATALINA_HOME in the script (startaim.sh or 
startaim.bat) that is used to start the server.  The recommended
location is /usr/local/tomcat (under Linux) or c:\\Tomcat (under
Windows)."],

 ['MapleProgram',
  string,"
The absolute path to the maple program.  Under Windows, the path
should use backslashes as separators, and should not contain any
spaces; in particular, 'Program Files' is disallowed.  The recommended
location is /usr/bin/maple (under Linux) or c:\\Maple\\bin.wnt\\cmaple 
(under Windows)."],

 ['MapleInitOptions',
  string,"
Initialisation options to be passed to Maple.  The value should be
\"default\" if you want the default options, which are as follows:

  kernelopts(printbytes=false): 
  interface(quiet=true, errorcursor=false, errorbreak=0):

There is no obvious reason to change this default."],

 ['TtHProgram',
  string,"
The absolute path to the tth program (used to convert LaTeX to HTML).
The command used by the server to start the TeX-to-HTML converter tth.
Under Windows, the path should use backslashes as separators, and
should not contain any spaces; in particular, 'Program Files' is
disallowed.  The recommended location is /usr/bin/tth (under Linux) or
c:\\TtH\\tth (under Windows)."],

# KM
 ['KillProgram',
  string,"
The absolute path to the kill program (used to kill Maple processes on this OS).
Under Windows, the path should use backslashes as separators, and
should not contain any spaces; in particular, 'Program Files' is
disallowed."],

 ['KillProgramArg',
  string,"
The argument for the kill program to tell it to kill the process whose PID
Is the final arg. Typically this is -k for the Process.exe program in Windows, and 
-9 for the kill command in UNIX."],

 ['AliceHistorySize',
  posint,"
The number of Recent Requests to show on the Alice Monitor page."],

 ['StudentTimeLimit',
  nonnegint,"
The time limit on non-administrative commands in milliseconds."],

 ['AdminTimeLimit',
  nonnegint,"
The time limit on administrative commands in milliseconds."],

 ['MaxMapleProcesses',
  posint,"
The maximum number of Maple processes that can be running at one time."],

# END KM

 ['TempDir',
  string,"
A directory for temporary storage of various things, including files 
uploaded by the servlet.  (The Maple part of the system will delete them
or move them to a more permanent home moments later.) Under Windows, 
the path should use backslashes as separators, and should not contain 
any spaces.  The default is /tmp (under Linux) or c:\\temp (under Windows)."],

 ['MaxPostSize',
  integer,"
The maximum allowed size for uploaded files, in bytes.  The default is
ten megabytes."],

 ['WEBINFDir',
  string,"
The WEB-INF subdirectory of the main AIM directory.  (All the private 
parts of the system are in this subdirectory; the parent directory and
its other subdirectories contain files that are publicly visible over
the web, such as the help file and all the images used by various AIM
questions.)  This should use forward slashes as separators, even under
windows."],

 ['TomcatPort',
  string,"
The number of the port on which Tomcat should listen.  This should 
normally be 80, which is the standard port for web servers.  The only
problem with this is if there is already another web server running on
the same machine and listening on port 80.  In that case, one would
normally use port 8000 or 8080.  The URL's used to access the servlet
will then look like http://.../Alice:8000, with the port number 
appearing explicitly on the end, which is not necessary if port 80 is
used."],

 ['TomcatContext',
  string,"
The name of the servlet context that contains the Alice servlet.
Normally this should be \"ROOT\"."],

 ['ServletLogFile',
  string,"
The full name of the servlet log file.  Under Windows, the path should 
use backslashes as separators, and should not contain any spaces."],

 ['LastRequestFile',
  string,"
The full name of the file in which the most recent request to the servlet
should be stored.  (This is very useful for debugging purposes.)  Under 
Windows, the path should use backslashes as separators, and should not 
contain any spaces."]
]:

TemporaryConfigFields := [
 'MapleHome',
 'TtHProgram',
 'HostName',
 'HostPort',
 'FullContext',
 'WebAppDir',
 'AimDir',
 'LogDir'
]:

interface(warnlevel = 2):

######################################################################

check_config_types :=
 proc()
  local f,key,typ,doc,val,msg;
  global Config;
  
  msg := "";
  for f in ConfigFields do
   key,typ,doc := op(f):
   if not(type([Config[key]],[typ])) then
    if Config[key] = NULL then
     msg :=
      sprintf(
       "%sConfig['%a'] should have type %a, but in fact it is NULL",
       msg,key,typ);
    elif traperror(member([key],[indices(Config)])) <> true then
     msg := 
      sprintf(
       "%sNo value has been assigned for Config['%a']",
       msg,key);
    else
     msg :=
      sprintf(
       "%sConfig['%a'] should have type %a, but in fact the value is %a",
       msg,key,typ,val);
    fi;
    msg := sprintf("%s\nDocumentation for this field:\n%s\n\n",msg,doc);
   fi;
  od;

  RETURN(msg);
 end:

make_blank_config :=
 proc()
  local f,key,typ,doc,val,msg;
  global Config;
  
  Config := table([]);
  for f in ConfigFields do
   key,typ,doc := op(f):
   if typ = string then
    Config[key] := "";
   elif typ = integer then
    Config[key] := 0;
   elif typ = boolean then
    Config[key] := false;
   fi;
  od;
 end:

######################################################################

check_servletconfig_types :=
 proc()
  local f,key,typ,doc,val,msg;
  global ServletConfig;
  
  msg := "";
  for f in ServletConfigFields do
   key,typ,doc := op(f):
   if not(type([ServletConfig[key]],[typ])) then
    if ServletConfig[key] = NULL then
     msg :=
      sprintf(
       "%sServletConfig['%a'] should have type %a, but in fact it is NULL",
       msg,key,typ);
    elif traperror(member([key],[indices(ServletConfig)])) <> true then
     msg := 
      sprintf(
       "%sNo value has been assigned for ServletConfig['%a']",
       msg,key);
    else
     msg :=
      sprintf(
       "%sServletConfig['%a'] should have type %a, but in fact the value is %a",
       msg,key,typ,val);
    fi;
    msg := sprintf("%s\nDocumentation for this field:\n%s\n\n",msg,doc);
   fi;
  od;

  RETURN(msg);
 end:

make_blank_servletconfig :=
 proc()
  local f,key,typ,doc,val,msg;
  global ServletConfig;
  
  ServletConfig := table([]);
  for f in ServletConfigFields do
   key,typ,doc := op(f):
   if typ = string then
    ServletConfig[key] := "";
   elif typ = integer then
    ServletConfig[key] := 0;
   elif typ = boolean then
    ServletConfig[key] := false;
   fi;
  od;
 end:


######################################################################
