# This script creates the servlet configuration file web.xml,
# and the scripts startaim.sh and stopaim.sh that start and
# stop the server.

read("OS0.mpl"):
read("Util0.mpl"):
read("ConfigFields.mpl"):
read("ServletConfig.mpl"):

######################################################################

err := check_servletconfig_types():

if err <> "" then
 printf("Configuration problems:\n%s\n",err):
 `quit`(1):
fi:

webinf := os_maplename(ServletConfig['WEBINFDir']):
 
fd := 
 traperror(fopen(cat(webinf,"/web.xml"),WRITE)):

if (fd = lasterror) then
 printf("Cannot open web.xml for writing:\n%a\n\n",fd):
 `quit`(1):
fi:

initparam := 
proc(nam::string,desc::string,default,key::name)
 global fd;
 local v;

 if nargs = 3 then
  v := default;
 elif not(assigned(ServletConfig[key])) or ServletConfig[key] = NULL then
  if default = 'complain' then
   if ServletConfig[key] = NULL then
    printf("Error: ServletConfig[%a] is NULL.\n",key):
   else
    printf("Error: ServletConfig[%a] is undefined.\n",key):
   fi;
   `quit`(1):
  else
   v := default;
  fi;
 elif type(ServletConfig[key],{integer,string,boolean,identical(omit)}) then
  v := ServletConfig[key];
 else
  printf(
  "Error: ServletConfig[%a] = %a, 
   which is not an integer, string or boolean value.\n",
   key,ServletConfig[key]):
  `quit`(1):  
 fi;

 if v <> 'omit' then
  fprintf(fd,"
    <init-param>
      <param-name>%s</param-name>
      <param-value>%A</param-value>
      <description>
       %s
      </description>
    </init-param>
  ",nam,v,desc);
 fi:
end:

fprintf(fd,"%s",
"<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>

<!DOCTYPE web-app
    PUBLIC \"-//Sun Microsystems, Inc.//DTD Web Application 2.3//EN\"
    \"http://java.sun.com/dtd/web-app_2_3.dtd\">

<!-- This is automatically generated from data in ServletConfig.mpl
     by running Servlet.mpl.  The file ServletConfig.mpl is in turn
     automatically generated by AutoConf.mpl, using values supplied
     in ManualConfig.mpl if necessary.
 -->

<web-app>
  <display-name>Alice Interactive Mathematics</display-name>
  <description>
   AIM is a system for computer-aided learning and assessment
   in mathematics.  Contact Neil Strickland
   (N.P.Strickland@sheffield.ac.uk) for more information.
  </description>

  <servlet>
    <servlet-name>Alice</servlet-name>
    <description>
    </description>
    <servlet-class>Alice</servlet-class>
"):

initparam(
 "AliceMapleDir",
 "The top directory for the AIM maple code.",
 cat(webinf,"/maple")
):

initparam(
 "TempDir",
 "The the directory used for temporary storage of uploaded files",
 'omit',
 'TempDir'
):

initparam(
 "MaxPostSize",
 "The the directory used for temporary storage of uploaded files",
 10 * 1024 * 1024,
 'MaxPostSize'
):

initparam(
 "MapleProgram",
 "The command used to start Maple",
 'omit',
 'MapleProgram'
):

initparam(
 "MapleInitOptions",
 "Initialization options for Maple processes started by the servlet",
 'omit',
 'MapleInitOptions'
):

initparam(
 "MaxMapleProcesses",
 "The maximum number of maple processes to be run by the server",
 8,
 'MaxMapleProcesses'
):

initparam(
 "BusyMessage",
 "The message to return if this count is exceeded.",
 "The Alice server is busy.  Please try again in a few minutes.",
 'BusyMessage'
):

initparam(
 "TtHProgram",
 "The command used to run  the TeX-to-HTML converter tth.",
 'omit',
 'TtHProgram'
):

# KM
initparam(
 "KillProgram",
 "The command used to kill processes on this OS.",
 'omit',
 'KillProgram'
):

initparam(
 "KillProgramArg",
 "The argument given to the kill command to kill processes on this OS (preceding the PID).",
 'omit',
 'KillProgramArg'
):

initparam(
 "AliceHistorySize",
 "The number of 'Recent Requests' to display on the Alice Monitor page.",
 20,
 'AliceHistorySize'
):

initparam(
 "StudentTimeLimit",
 "The time limit on non-administrative commands in milliseconds.",
 60000,
 'StudentTimeLimit'
):

initparam(
 "AdminTimeLimit",
 "The time limit on administrative commands in milliseconds.",
 1200000,
 'AdminTimeLimit'
):

# END KM

initparam(
 "LogFile",
 "The file for logging all servlet requests.",
 'omit',
 'ServletLogFile'
):

initparam(
 "LastRequestFile",
 "The file for logging the last servlet request.",
 'omit',
 'LastRequestFile'
):

initparam(
 "TtHLogFile",
 "The file for logging problems with TtH.",
 'omit',
 'TtHLogFile'
):

initparam(
 "TtHLogLevel",
 "The level of logging for problems with TtH.",
 'omit',
 'TtHLogLevel'
):

fprintf(fd,"%s","
    <load-on-startup>5</load-on-startup>
  </servlet>

  <servlet-mapping>
    <servlet-name>Alice</servlet-name>
    <url-pattern>/Alice</url-pattern>
  </servlet-mapping>
  <session-config>
    <session-timeout>30</session-timeout>    <!-- 30 minutes -->
  </session-config>
</web-app>
"):

fclose(fd):

######################################################################

file := cat(webinf,"/server.xml"):

fd := traperror(fopen(file,'WRITE')):

if fd = lasterror then
 printf("Cannot open web.xml for writing:\n%a\n\n",fd):
 `quit`(1):
fi:

port := ServletConfig['TomcatPort']:
ctxt := ServletConfig['TomcatContext']:
path := `if`(ctxt = "ROOT","",ctxt):

fprintf(fd,"
<!-- AIM Server Configuration File -->

<!-- This is automatically generated from data in ServletConfig.mpl
     by running Servlet.mpl.  The file ServletConfig.mpl is in turn
     automatically generated by AutoConf.mpl, using values supplied
     in ManualConfig.mpl if necessary.
 -->

<Server port=\"8005\" shutdown=\"SHUTDOWN\" debug=\"0\">
  <Service name=\"Tomcat-Standalone\">
    <Connector className=\"org.apache.catalina.connector.http.HttpConnector\"
               port=\"%s\" minProcessors=\"5\" maxProcessors=\"75\"
               enableLookups=\"true\" redirectPort=\"8443\"
               acceptCount=\"10\" debug=\"0\" connectionTimeout=\"60000\"/>
    <Engine name=\"Standalone\" defaultHost=\"localhost\" debug=\"0\">
      <Logger className=\"org.apache.catalina.logger.FileLogger\"
              prefix=\"catalina_log.\" suffix=\".txt\"
              timestamp=\"true\"/>
      <Realm className=\"org.apache.catalina.realm.MemoryRealm\" />
      <Host name=\"localhost\" debug=\"0\"
             appBase=\"webapps\" unpackWARs=\"true\">
        <Valve className=\"org.apache.catalina.valves.AccessLogValve\"
                 directory=\"logs\"  prefix=\"access.\" suffix=\".txt\"
                 pattern=\"common\"/>
        <Logger className=\"org.apache.catalina.logger.FileLogger\"
                 directory=\"logs\"  prefix=\"log.\" suffix=\".txt\"
         timestamp=\"true\"/>
        <Context path=\"%s\" docBase=\"%s\" debug=\"0\" reloadable=\"true\"/>
      </Host>
    </Engine>
  </Service>
</Server>

",port,path,ctxt

):

unassign('port','ctxt','dbse'):
fclose(fd):

file1 := 
 os_maplename(cat(ServletConfig['CatalinaHome'],"/conf/server.xml")):
file2 := 
 os_maplename(cat(ServletConfig['CatalinaHome'],"/conf/server.bak")):

if os_fileexists(file1) then
 printf("Backing up %s\n",file1):
 err := traperror(os_copy(file1,file2)):
 if err = lasterror then
  printf("Failed: %s\n\n",err):
  `quit`(1):
 fi:
fi: 

printf("Copying %s to %s\n", file, file1):
err := traperror(os_copy(file,file1)):
if err = lasterror then
 printf("Failed: %s\n\n",err):
 `quit`(1):
fi:

######################################################################

if os_os = "Linux" then
 file := cat(webinf,"/startaim.sh"):
 fd := traperror(fopen(file,WRITE)):
 if fd = lasterror then
  printf("Cannot open startaim.sh for writing:\n%a\n\n",fd):
  `quit`(1):
 fi:

 fprintf(fd,
 "#!/bin/sh
 JAVA_HOME=%s
 export JAVA_HOME
 CATALINA_HOME=%s
 export CATALINA_HOME
 %s/bin/startup.sh
 ",
 ServletConfig['JavaHome'],
 ServletConfig['CatalinaHome'],
 ServletConfig['CatalinaHome']
 ):

 fclose(fd):

 system(sprintf("chown nobody:aim %s",file)):
 system(sprintf("chmod 774 %s",file)):

 file := cat(webinf,"/stopaim.sh"):
 fd := traperror(fopen(file,WRITE)):
 if fd = lasterror then
  printf("Cannot open stopaim.sh for writing:\n%a\n\n",fd):
  `quit`(1):
 fi:

 fprintf(fd,
 "#!/bin/sh
 JAVA_HOME=%s
 export JAVA_HOME
 CATALINA_HOME=%s
 export CATALINA_HOME
 %s/bin/shutdown.sh
 ",
 ServletConfig['JavaHome'],
 ServletConfig['CatalinaHome'],
 ServletConfig['CatalinaHome']
 ):

 fclose(fd):

 system(sprintf("chown nobody:aim %s",file)):
 system(sprintf("chmod 774 %s",file)):
elif os_os = "DOS" then
 file := cat(webinf,"/startaim.bat"):
 fd := traperror(fopen(file,WRITE)):
 if fd = lasterror then
  printf("Cannot open startaim.bat for writing:\n%a\n\n",fd):
  `quit`(1):
 fi:

 fprintf(fd,
 "@echo off
SET JAVA_HOME=%s
SET CATALINA_HOME=%s
call %s\\bin\\startup.bat
 ",
 ServletConfig['JavaHome'],
 ServletConfig['CatalinaHome'],
 ServletConfig['CatalinaHome']
 ):

 fclose(fd):

 file := cat(webinf,"/stopaim.bat"):
 fd := traperror(fopen(file,WRITE)):
 if fd = lasterror then
  printf("Cannot open stopaim.bat for writing:\n%a\n\n",fd):
  `quit`(1):
 fi:

 fprintf(fd,
 "@echo off
SET JAVA_HOME=%s
SET CATALINA_HOME=%s
call %s\\bin\\shutdown.bat
 ",
 ServletConfig['JavaHome'],
 ServletConfig['CatalinaHome'],
 ServletConfig['CatalinaHome']
 ):

 fclose(fd):

fi:

######################################################################

# Copy the com.oreilly.servlet classes

cosdir    := "com/oreilly/servlet/":
cossource := cat("../cos/",cosdir):

# Of the three lines below, the first is approved for Tomcat 4.0 but
# fails for Tomcat 4.1, and the reverse is true of the second.  The
# third option is not really the approved place for general purpose
# classes, but it works with all versions of Tomcat.
#cosdest   := cat(ServletConfig['CatalinaHome'],"/classes/",cosdir):
#cosdest   := cat(ServletConfig['CatalinaHome'],"/shared/classes/",cosdir):
cosdest   := cat(webinf,"/classes/",cosdir):

if not(os_fileexists(cat(cosdest,"/multipart/Part.class"))) then
 err := traperror(os_makedirectory(cat(cosdest,"/multipart"))):

 if err = lasterror then
  printf("Could not create directory for com.oreilly.servlet classes\n"):
  `quit`(1):
 fi:

 for f in os_listfiles(cossource) do
  if f <> "multipart" then
   os_copy(cat(cossource,"/",f),cat(cosdest,"/",f)):
  fi:
 od:

 for f in os_listfiles(cat(cossource,"/multipart")) do
  os_copy(cat(cossource,"/multipart/",f),
   cat(cosdest,"/multipart/",f)):
 od:

 if not(os_fileexists(cat(cosdest,"/multipart/Part.class"))) then
  printf("Could not copy com.oreilly.servlet classes\n"):
  `quit`(1):
 fi:
else
 printf("com.oreilly.servlet classes are already installed.\n"):
fi: